@@ -25,7 +25,7 @@
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE. */
-/* $Id: boot.h,v 1.12.4.3 2005/07/29 21:43:36 joerg_wunsch Exp $ */
+/* $Id: boot.h,v 1.12.4.6 2005/10/06 20:06:41 aesok Exp $ */
 
 #ifndef _AVR_BOOT_H_
 #define _AVR_BOOT_H_    1
@@ -189,8 +189,9 @@
         "sts %0, %1\n\t"                         \
         "spm\n\t"                                \
         "clr  r1\n\t"                            \
-        : "=m" (__SPM_REG)                       \
-        : "r" ((uint8_t)__BOOT_PAGE_FILL),       \
+        :                                        \
+        : "i" (_SFR_MEM_ADDR(__SPM_REG)),        \
+          "r" ((uint8_t)__BOOT_PAGE_FILL),       \
           "r" ((uint16_t)address),               \
           "r" ((uint16_t)data)                   \
         : "r0", "r30", "r31"                     \
@@ -208,8 +209,9 @@
         ".word 0xffff\n\t"                       \
         "nop\n\t"                                \
         "clr  r1\n\t"                            \
-        : "=m" (__SPM_REG)                       \
-        : "r" ((uint8_t)__BOOT_PAGE_FILL),       \
+        :                                        \
+        : "i" (_SFR_MEM_ADDR(__SPM_REG)),        \
+          "r" ((uint8_t)__BOOT_PAGE_FILL),       \
           "r" ((uint16_t)address),               \
           "r" ((uint16_t)data)                   \
         : "r0", "r30", "r31"                     \
@@ -226,9 +228,10 @@
         "sts %0, %2\n\t"                         \
         "spm\n\t"                                \
         "clr  r1\n\t"                            \
-        : "=m" (__SPM_REG),                      \
-          "=m" (RAMPZ)                           \
-        : "r" ((uint8_t)__BOOT_PAGE_FILL),       \
+        :                                        \
+        : "i" (_SFR_MEM_ADDR(__SPM_REG)),        \
+          "i" (_SFR_MEM_ADDR(RAMPZ)),            \
+          "r" ((uint8_t)__BOOT_PAGE_FILL),       \
           "r" ((uint32_t)address),               \
           "r" ((uint16_t)data)                   \
         : "r0", "r30", "r31"                     \
@@ -242,8 +245,9 @@
         "movw r30, %2\n\t"                       \
         "sts %0, %1\n\t"                         \
         "spm\n\t"                                \
-        : "=m" (__SPM_REG)                       \
-        : "r" ((uint8_t)__BOOT_PAGE_ERASE),      \
+        :                                        \
+        : "i" (_SFR_MEM_ADDR(__SPM_REG)),        \
+          "r" ((uint8_t)__BOOT_PAGE_ERASE),      \
           "r" ((uint16_t)address)                \
         : "r30", "r31"                           \
     );                                           \
@@ -258,8 +262,9 @@
         "spm\n\t"                                \
         ".word 0xffff\n\t"                       \
         "nop\n\t"                                \
-        : "=m" (__SPM_REG)                       \
-        : "r" ((uint8_t)__BOOT_PAGE_ERASE),      \
+        :                                        \
+        : "i" (_SFR_MEM_ADDR(__SPM_REG)),        \
+          "r" ((uint8_t)__BOOT_PAGE_ERASE),      \
           "r" ((uint16_t)address)                \
         : "r30", "r31"                           \
     );                                           \
@@ -273,9 +278,10 @@
         "sts  %1, %C3\n\t"                       \
         "sts %0, %2\n\t"                         \
         "spm\n\t"                                \
-        : "=m" (__SPM_REG),                      \
-          "=m" (RAMPZ)                           \
-        : "r" ((uint8_t)__BOOT_PAGE_ERASE),      \
+        :                                        \
+        : "i" (_SFR_MEM_ADDR(__SPM_REG)),        \
+          "i" (_SFR_MEM_ADDR(RAMPZ)),            \
+          "r" ((uint8_t)__BOOT_PAGE_ERASE),      \
           "r" ((uint32_t)address)                \
         : "r30", "r31"                           \
     );                                           \
@@ -288,8 +294,9 @@
         "movw r30, %2\n\t"                       \
         "sts %0, %1\n\t"                         \
         "spm\n\t"                                \
-        : "=m" (__SPM_REG)                       \
-        : "r" ((uint8_t)__BOOT_PAGE_WRITE),      \
+        :                                        \
+        : "i" (_SFR_MEM_ADDR(__SPM_REG)),        \
+          "r" ((uint8_t)__BOOT_PAGE_WRITE),      \
           "r" ((uint16_t)address)                \
         : "r30", "r31"                           \
     );                                           \
@@ -304,8 +311,9 @@
         "spm\n\t"                                \
         ".word 0xffff\n\t"                       \
         "nop\n\t"                                \
-        : "=m" (__SPM_REG)                       \
-        : "r" ((uint8_t)__BOOT_PAGE_WRITE),      \
+        :                                        \
+        : "i" (_SFR_MEM_ADDR(__SPM_REG)),        \
+          "r" ((uint8_t)__BOOT_PAGE_WRITE),      \
           "r" ((uint16_t)address)                \
         : "r30", "r31"                           \
     );                                           \
@@ -319,9 +327,10 @@
         "sts %1, %C3\n\t"                        \
         "sts %0, %2\n\t"                         \
         "spm\n\t"                                \
-        : "=m" (__SPM_REG),                      \
-          "=m" (RAMPZ)                           \
-        : "r" ((uint8_t)__BOOT_PAGE_WRITE),      \
+        :                                        \
+        : "i" (_SFR_MEM_ADDR(__SPM_REG)),        \
+          "i" (_SFR_MEM_ADDR(RAMPZ)),            \
+          "r" ((uint8_t)__BOOT_PAGE_WRITE),      \
           "r" ((uint32_t)address)                \
         : "r30", "r31"                           \
     );                                           \
@@ -333,8 +342,9 @@
     (                                            \
         "sts %0, %1\n\t"                         \
         "spm\n\t"                                \
-        : "=m" (__SPM_REG)                       \
-        : "r" ((uint8_t)__BOOT_RWW_ENABLE)       \
+        :                                        \
+        : "i" (_SFR_MEM_ADDR(__SPM_REG)),        \
+          "r" ((uint8_t)__BOOT_RWW_ENABLE)       \
     );                                           \
 })
 
@@ -346,8 +356,9 @@
         "spm\n\t"                                \
         ".word 0xffff\n\t"                       \
         "nop\n\t"                                \
-        : "=m" (__SPM_REG)                       \
-        : "r" ((uint8_t)__BOOT_RWW_ENABLE)       \
+        :                                        \
+        : "i" (_SFR_MEM_ADDR(__SPM_REG)),        \
+          "r" ((uint8_t)__BOOT_RWW_ENABLE)       \
     );                                           \
 })
 
@@ -378,8 +389,9 @@
         "mov r0, %2\n\t"                                   \
         "sts %0, %1\n\t"                                   \
         "spm\n\t"                                          \
-        : "=m" (__SPM_REG)                                 \
-        : "r" ((uint8_t)__BOOT_LOCK_BITS_SET),             \
+        :                                                  \
+        : "i" (_SFR_MEM_ADDR(__SPM_REG)),                  \
+          "r" ((uint8_t)__BOOT_LOCK_BITS_SET),             \
           "r" (value)                                      \
         : "r0", "r30", "r31"                               \
     );                                                     \
@@ -397,8 +409,9 @@
         "spm\n\t"                                          \
         ".word 0xffff\n\t"                                 \
         "nop\n\t"                                          \
-        : "=m" (__SPM_REG)                                 \
-        : "r" ((uint8_t)__BOOT_LOCK_BITS_SET),       \
+        :                                                  \
+        : "i" (_SFR_MEM_ADDR(__SPM_REG)),                  \
+          "r" ((uint8_t)__BOOT_LOCK_BITS_SET),             \
           "r" (value)                                      \
         : "r0", "r30", "r31"                               \
     );                                                     \
@@ -457,22 +470,22 @@
     is programmed.
  */
 #define boot_lock_fuse_bits_get(address)                   \
-({                                                         \
+(__extension__({                                           \
     uint8_t __result;                                      \
     __asm__ __volatile__                                   \
     (                                                      \
         "ldi r30, %3\n\t"                                  \
         "ldi r31, 0\n\t"                                   \
-        "sts %0, %2\n\t"                                   \
-        "lpm %1, Z\n\t"                                    \
-        : "=m" (__SPM_REG),                                \
-          "=r" (__result)                                  \
-        : "r" ((uint8_t)__BOOT_LOCK_BITS_SET),             \
+        "sts %1, %2\n\t"                                   \
+        "lpm %0, Z\n\t"                                    \
+        : "=r" (__result)                                  \
+        : "i" (_SFR_MEM_ADDR(__SPM_REG)),                  \
+          "r" ((uint8_t)__BOOT_LOCK_BITS_SET),             \
           "M" (address)                                    \
         : "r0", "r30", "r31"                               \
     );                                                     \
     __result;                                              \
-})
+}))
 
 /** \ingroup avr_boot
     \def boot_page_fill(address, data)
@@ -565,51 +578,64 @@
 
 #endif
 
-#define __boot_eeprom_spm_safe(func, address, data) \
-do { \
-    boot_spm_busy_wait();                       \
-    eeprom_busy_wait();                         \
-    func (address, data);                       \
-} while (0)
-
 /** \ingroup avr_boot
 
     Same as boot_page_fill() except it waits for eeprom and spm operations to
     complete before filling the page. */
 
 #define boot_page_fill_safe(address, data) \
-    __boot_eeprom_spm_safe (boot_page_fill, address, data)
+do { \
+    boot_spm_busy_wait();                       \
+    eeprom_busy_wait();                         \
+    boot_page_fill(address, data);              \
+} while (0)
 
 /** \ingroup avr_boot
 
     Same as boot_page_erase() except it waits for eeprom and spm operations to
     complete before erasing the page. */
 
-#define boot_page_erase_safe(address, data) \
-    __boot_eeprom_spm_safe (boot_page_erase, address, data)
+#define boot_page_erase_safe(address) \
+do { \
+    boot_spm_busy_wait();                       \
+    eeprom_busy_wait();                         \
+    boot_page_erase (address);                  \
+} while (0)
 
 /** \ingroup avr_boot
 
     Same as boot_page_write() except it waits for eeprom and spm operations to
     complete before writing the page. */
 
-#define boot_page_write_safe(address, data) \
-    __boot_eeprom_spm_safe (boot_page_wrte, address, data)
+#define boot_page_write_safe(address) \
+do { \
+    boot_spm_busy_wait();                       \
+    eeprom_busy_wait();                         \
+    boot_page_write (address);                  \
+} while (0)
 
 /** \ingroup avr_boot
 
     Same as boot_rww_enable() except waits for eeprom and spm operations to
     complete before enabling the RWW mameory. */
 
-#define boot_rww_enable_safe(address, data) \
-    __boot_eeprom_spm_safe (boot_rww_enable, address, data)
+#define boot_rww_enable_safe() \
+do { \
+    boot_spm_busy_wait();                       \
+    eeprom_busy_wait();                         \
+    boot_rww_enable();                          \
+} while (0)
 
 /** \ingroup avr_boot
 
     Same as boot_lock_bits_set() except waits for eeprom and spm operations to
     complete before setting the lock bits. */
 
-#define boot_lock_bits_set_safe(address, data) \
-    __boot_eeprom_spm_safe (boot_lock_bits_set, address, data)
+#define boot_lock_bits_set_safe(lock_bits) \
+do { \
+    boot_spm_busy_wait();                       \
+    eeprom_busy_wait();                         \
+    boot_lock_bits_set (lock_bits);             \
+} while (0)
 
 #endif /* _AVR_BOOT_H_ */
