@@ -28,7 +28,7 @@
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE. */
 
-/* $Id: iom325.h,v 1.2.2.4 2005/07/31 21:02:49 joerg_wunsch Exp $ */
+/* $Id: iom325.h,v 1.2.2.5 2005/10/16 11:42:31 aesok Exp $ */
 
 /* avr/iom325.h - definitions for ATmega325 */
 
@@ -151,11 +151,11 @@
 
 /* Reserved [0x28..0x29] */
 
-#define GPIOR1  SFR_IO8(0x2A)
+#define GPIOR1  _SFR_IO8(0x2A)
 
-#define GPIOR2  SFR_IO8(0x2B)
+#define GPIOR2  _SFR_IO8(0x2B)
 
-#define SPCR    SFR_IO8(0x2C)
+#define SPCR    _SFR_IO8(0x2C)
 #define SPR0    0
 #define SPR1    1
 #define CPHA    2
@@ -165,16 +165,16 @@
 #define SPE     6
 #define SPIE    7
 
-#define SPSR    SFR_IO8(0x2D)
+#define SPSR    _SFR_IO8(0x2D)
 #define SPI2X   0
 #define WCOL    6
 #define SPIF    7
 
-#define SPDR    SFR_IO8(0X2E)
+#define SPDR    _SFR_IO8(0X2E)
 
 /* Reserved [0x2F] */
 
-#define ACSR    SFR_IO8(0x30)
+#define ACSR    _SFR_IO8(0x30)
 #define ACIS0   0
 #define ACIS1   1
 #define ACIC    2
@@ -184,7 +184,7 @@
 #define ACBG    6
 #define ACD     7
 
-#define OCDR    SFR_IO8(0x31)
+#define OCDR    _SFR_IO8(0x31)
 #define OCDR0   0
 #define OCDR1   1
 #define OCDR2   2
