@@ -28,7 +28,7 @@
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE. */
 
-/* $Id: iotn13.h,v 1.3 2004/11/02 18:16:07 arcanum Exp $ */
+/* $Id: iotn13.h,v 1.3.2.1 2005/07/19 15:02:13 aesok Exp $ */
 
 /* avr/iotn13.h - definitions for ATtiny13 */
 
@@ -147,9 +147,21 @@
 #  define PB1                  1
 #  define PB0                  0
 
-/* These are not defined in <avr/io.h> like the rest of the eeprom names. */
-#  define EEPM1                5
-#  define EEPM0                4
+/* ATtiny EEPROM Control Register EECR */
+#define EECR	             _SFR_IO8(0x1C)
+#define EEPM1                  5
+#define EEPM0                  4
+#define EERIE                  3
+#define EEMPE                  2
+#define EEPE                   1
+#define EERE                   0
+
+/* EEPROM Data Register */
+#define EEDR	             _SFR_IO8(0x1D)
+
+/* The EEPROM Address Register EEAR[6:0] */
+#define EEAR	             _SFR_IO8(0x1E)
+#define EEARL	             _SFR_IO8(0x1E)
 
 /* Watchdog Timer Control Register */
 #define WDTCR                _SFR_IO8(0x21)
