@@ -1,5 +1,6 @@
 /* Copyright (c) 2002, 2004 Theodore A. Roth
    Copyright (c) 2004 Eric B. Weddington
+   Copyright (c) 2005 Joerg Wunsch
    All rights reserved.
 
    Redistribution and use in source and binary forms, with or without
@@ -29,7 +30,7 @@
    ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
    POSSIBILITY OF SUCH DAMAGE. */
 
-/* $Id: sleep.h,v 1.9 2004/12/30 00:29:18 arcanum Exp $ */
+/* $Id: sleep.h,v 1.9.2.2 2005/09/10 21:48:56 joerg_wunsch Exp $ */
 
 #ifndef _AVR_SLEEP_H_
 #define _AVR_SLEEP_H_ 1
@@ -76,6 +77,10 @@
 
 #define _SLEEP_CONTROL_REG SMCR
 
+#elif defined(__AVR_AT94K__)
+
+#define _SLEEP_CONTROL_REG MCUR
+
 #else
 
 #define _SLEEP_CONTROL_REG MCUCR
@@ -110,14 +115,36 @@
 
 
 #if _SLEEP_TYPE == 2
+
+/*
+ * Type 2 devices are not completely identical, so we need a few
+ * #ifdefs here.
+ *
+ * Note that it appears the datasheet of the tiny2313 has the bottom
+ * two lines of table 13 with the wrong SM0/SM1 values.
+ */
 #define SLEEP_MODE_IDLE         0
-#define SLEEP_MODE_ADC          _BV(SM0)
+
+#if !defined(__AVR_ATtiny2313__) && !defined(__AVR_AT94K__)
+/* no ADC in ATtiny2313, SM0 is alternative powerdown mode */
+/* no ADC in AT94K, setting SM0 only is reserved */
+# define SLEEP_MODE_ADC          _BV(SM0)
+#endif /* !defined(__AVR_ATtiny2313__) && !defined(__AVR_AT94K__) */
+
 #define SLEEP_MODE_PWR_DOWN     _BV(SM1)
-#define SLEEP_MODE_PWR_SAVE     (_BV(SM0) | _BV(SM1))
+
+#if defined(__AVR_ATtiny2313__) || defined(__AVR_ATtiny26__)
+/* tiny2313 and tiny26 have standby rather than powersave */
+# define SLEEP_MODE_STANDBY      (_BV(SM0) | _BV(SM1))
+#elif !defined(__AVR_ATtiny13__)
+/* SM0|SM1 is reserved on the tiny13 */
+# define SLEEP_MODE_PWR_SAVE     (_BV(SM0) | _BV(SM1))
+#endif
+
 #endif
 
 
-#if _SLEEP_TYPE == 3
+#if _SLEEP_TYPE == 3 || defined(DOXYGEN)
 /** \ingroup avr_sleep
     \def SLEEP_MODE_IDLE
     Idle mode. */
